@@ -29,7 +29,9 @@ import uvicorn
 from llama_cpp.server.app import create_app, Settings
 
 if __name__ == "__main__":
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter
+    )
     for name, field in Settings.__fields__.items():
         parser.add_argument(
             f"--{name}",
@@ -40,7 +42,7 @@ if __name__ == "__main__":
         )
 
     args = parser.parse_args()
-    settings = Settings(**vars(args))
+    settings = Settings(**{k: v for k, v in vars(args).items() if v is not None})
     app = create_app(settings=settings)
 
     uvicorn.run(
